﻿Imports System.Windows.Forms, System.Collections, System.Drawing
Public Class ControlStyling

#Region "TextBox"

	''' <summary> Überschreibt das eingestellte "Styling" der TextBox </summary>
	Public Shared Sub StyleTb(frm As ContainerControl)
		Dim getChilds As Func(Of Control, IEnumerable) = Function(ctl) ctl.Controls '.Controls
		For Each ctl In getChilds.All(frm)
			Dim tb = TryCast(ctl, TextBox)
			If tb.NotNull Then
				' man kann hier auch andere Controls stylen, nicht nur DGV
			End If
		Next
	End Sub

#End Region 'TextBox

#Region "ComboBox"

	''' <summary> Überschreibt das eingestellte "Styling" aller ComboBoxen auf der Form </summary>
	Public Shared Sub StyleCmbList(frm As ContainerControl)
		Dim getChilds As Func(Of Control, IEnumerable) = Function(ctl) ctl.Controls '.Controls
		For Each ctl In getChilds.All(frm)
			Dim cmb = TryCast(ctl, ComboBox)
			If cmb.NotNull Then
				With cmb
					.DropDownStyle = ComboBoxStyle.DropDownList
					.AutoCompleteSource = AutoCompleteSource.ListItems
					.AutoCompleteMode = AutoCompleteMode.SuggestAppend
				End With
			End If
		Next
	End Sub

	Public Shared Sub StyleCmbList(ParamArray cmbs As ComboBox())
		For Each cmb In cmbs
			With cmb
				.DropDownStyle = ComboBoxStyle.DropDownList
				.AutoCompleteSource = AutoCompleteSource.ListItems
				.AutoCompleteMode = AutoCompleteMode.SuggestAppend
			End With
		Next
	End Sub

	''' <summary> Styled einzelne ComboBoxen mit DropDownStyle = Dropdown</summary>
	Public Shared Sub StyleCmbFreetext(ParamArray cmbs As ComboBox())
		For Each cmb In cmbs
			With cmb
				.DropDownStyle = ComboBoxStyle.DropDown
				.AutoCompleteSource = AutoCompleteSource.ListItems
				.AutoCompleteMode = AutoCompleteMode.SuggestAppend
			End With
		Next
	End Sub

	''' <summary> Styled alle im Form enthaltenen ComboBoxen mit DropDownStyle = Dropdown</summary>
	Public Shared Sub StyleCmbFreetext(frm As ContainerControl)
		Dim getChilds As Func(Of Control, IEnumerable) = Function(ctl) ctl.Controls '.Controls
		For Each ctl In getChilds.All(frm)
			Dim cmb = TryCast(ctl, ComboBox)
			If cmb.NotNull Then
				With cmb
					.DropDownStyle = ComboBoxStyle.DropDown
					.AutoCompleteSource = AutoCompleteSource.ListItems
					.AutoCompleteMode = AutoCompleteMode.SuggestAppend
				End With
			End If
		Next
	End Sub

#End Region 'ComboBox

#Region "DGV"

	''' <summary> Überschreibt das eingestellte "Styling" des DataGridView </summary>
	Public Shared Sub StyleDGV(frm As ContainerControl)
		Dim getChilds As Func(Of Control, IEnumerable) = Function(ctl) ctl.Controls '.Controls
		For Each ctl In getChilds.All(frm)
			Dim grd = TryCast(ctl, DataGridView)
			If grd.NotNull Then
				Dim tbl = DirectCast(grd.DataSource, BindingSource)?.DataTable
				If tbl.NotNull Then
					For Each dgvCol As DataGridViewColumn In grd.Columns
						Dim propName = dgvCol.DataPropertyName
						If propName = "" Then Continue For
						Dim dataCol = tbl.Columns(propName)
						If dataCol Is Nothing Then
							msgError($"Die Spalte {propName} existiert nicht.{Environment.NewLine}DGV:{grd.Name}{Environment.NewLine}Tabelle:{tbl.TableName}{Environment.NewLine}Form:{frm.Name}")
							Return
						End If
						If dataCol.ReadOnly = True Then dgvCol.DefaultCellStyle.BackColor = SystemColors.Menu
						If dataCol.DataType = GetType(Decimal) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
								'.Format = "c"
								'.Font = New Font("Courier New", If(.Font, If(grd.DefaultCellStyle.Font, grd.Font)).Size)
							End With
						ElseIf dataCol.DataType = GetType(Double) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
								'      .Format = "n2"
								'      .Font = New Font("Courier New", If(.Font, If(grd.DefaultCellStyle.Font, grd.Font)).Size)
							End With
						ElseIf dataCol.DataType = GetType(DateTime) Then
							'With dgvCol.DefaultCellStyle
							'	.Format = "dd.MM.yyyy"
							'End With
						ElseIf dataCol.DataType = GetType(TimeSpan) Then
							With dgvCol.DefaultCellStyle
								.Format = "hh\:mm"
							End With
						ElseIf dataCol.DataType = GetType(Int32) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
							End With
						ElseIf dataCol.DataType = GetType(System.String) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
							End With
						End If
					Next
				Else
					For Each dgvCol In grd.ColumnsX
						If dgvCol.ValueType = GetType(Decimal) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
						If dgvCol.ValueType = GetType(Double) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
						If dgvCol.ValueType = GetType(Integer) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
					Next
				End If
				With grd
					.AllowUserToResizeRows = False
					.AllowUserToResizeColumns = True
					.RowHeadersWidth = 25
					.Attach.UnFocusedSelectionColor()
					'.CellBorderStyle = DataGridViewCellBorderStyle.SingleHorizontal
					.CellBorderStyle = DataGridViewCellBorderStyle.Raised
					.SelectionMode = DataGridViewSelectionMode.FullRowSelect
					.ShowEditingIcon = False
					.RowHeadersVisible = False
					.BackgroundColor = SystemColors.Window
					'Designer-Aufgaben-Einstellung
					.AllowUserToAddRows = False
					.ReadOnly = True
					.AllowUserToDeleteRows = False
					.AllowUserToOrderColumns = False
				End With
				AddHandler grd.DataError, AddressOf DgvErrorHandling
			End If
		Next
	End Sub

	Public Shared Sub StyleDGVlv(dgv As DataGridView)
		Dim tbl = DirectCast(dgv.DataSource, BindingSource)?.DataTable
		If tbl.NotNull Then
			For Each dgvCol In dgv.ColumnsX
				Dim propName = dgvCol.DataPropertyName
				If propName = "" Then Continue For
				Dim dataCol = tbl.Columns(propName)
				If dataCol.ReadOnly = True Then dgvCol.DefaultCellStyle.BackColor = SystemColors.Menu
				If dataCol.DataType = GetType(Decimal) Then
					With dgvCol.DefaultCellStyle
						.Alignment = DataGridViewContentAlignment.MiddleLeft
						'.Format = "c"
						'.Font = New Font("Courier New", If(.Font, If(grd.DefaultCellStyle.Font, grd.Font)).Size)
					End With
				ElseIf dataCol.DataType = GetType(Double) Then
					With dgvCol.DefaultCellStyle
						.Alignment = DataGridViewContentAlignment.MiddleLeft
						'      .Format = "n2"
						'      .Font = New Font("Courier New", If(.Font, If(grd.DefaultCellStyle.Font, grd.Font)).Size)
					End With
				ElseIf dataCol.DataType = GetType(DateTime) Then
					'With dgvCol.DefaultCellStyle
					'	.Format = "dd.MM.yyyy"
					'End With
				ElseIf dataCol.DataType = GetType(TimeSpan) Then
					With dgvCol.DefaultCellStyle
						.Format = "hh\:mm"
					End With
				ElseIf dataCol.DataType = GetType(Int32) Then
					With dgvCol.DefaultCellStyle
						.Alignment = DataGridViewContentAlignment.MiddleLeft
					End With
				ElseIf dataCol.DataType = GetType(System.String) Then
					With dgvCol.DefaultCellStyle
						.Alignment = DataGridViewContentAlignment.MiddleLeft
					End With
				End If
			Next
		Else
			For Each dgvCol In dgv.ColumnsX
				If dgvCol.ValueType = GetType(Decimal) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
				If dgvCol.ValueType = GetType(Double) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
				If dgvCol.ValueType = GetType(Integer) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
			Next
		End If
		With dgv
			.AllowUserToResizeRows = False
			.AllowUserToResizeColumns = True
			.RowHeadersWidth = 25
			.SelectionMode = DataGridViewSelectionMode.FullRowSelect
			.ShowEditingIcon = False
			.RowHeadersVisible = False
			.BackgroundColor = SystemColors.Window
			'Designer-Aufgaben-Einstellung
			.AllowUserToAddRows = False
			.ReadOnly = True
			.AllowUserToDeleteRows = False
			.AllowUserToOrderColumns = False
		End With
		AddHandler dgv.DataError, AddressOf DgvErrorHandling
	End Sub

	''' <summary> Überschreibt das eingestellte "Styling" des DataGridView. Orientiert sich am ListView-Design </summary>
	Public Shared Sub StyleDGVlv(frm As ContainerControl)
		Dim getChilds As Func(Of Control, IEnumerable) = Function(ctl) ctl.Controls '.Controls
		For Each ctl In getChilds.All(frm)
			Dim grd = TryCast(ctl, DataGridView)
			If grd.NotNull Then
				Dim tbl = DirectCast(grd.DataSource, BindingSource)?.DataTable
				If tbl.NotNull Then
					For Each dgvCol As DataGridViewColumn In grd.Columns
						Dim propName = dgvCol.DataPropertyName
						If propName = "" Then Continue For
						Dim dataCol = tbl.Columns(propName)
						If dataCol.ReadOnly = True Then dgvCol.DefaultCellStyle.BackColor = SystemColors.Menu
						If dataCol.DataType = GetType(Decimal) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
								'.Format = "c"
								'.Font = New Font("Courier New", If(.Font, If(grd.DefaultCellStyle.Font, grd.Font)).Size)
							End With
						ElseIf dataCol.DataType = GetType(Double) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
								'      .Format = "n2"
								'      .Font = New Font("Courier New", If(.Font, If(grd.DefaultCellStyle.Font, grd.Font)).Size)
							End With
						ElseIf dataCol.DataType = GetType(DateTime) Then
							'With dgvCol.DefaultCellStyle
							'	.Format = "dd.MM.yyyy"
							'End With
						ElseIf dataCol.DataType = GetType(TimeSpan) Then
							With dgvCol.DefaultCellStyle
								.Format = "hh\:mm"
							End With
						ElseIf dataCol.DataType = GetType(Int32) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
							End With
						ElseIf dataCol.DataType = GetType(System.String) Then
							With dgvCol.DefaultCellStyle
								.Alignment = DataGridViewContentAlignment.MiddleLeft
							End With
						End If
					Next
				Else
					For Each dgvCol In grd.ColumnsX
						If dgvCol.ValueType = GetType(Decimal) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
						If dgvCol.ValueType = GetType(Double) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
						If dgvCol.ValueType = GetType(Integer) Then dgvCol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
					Next
				End If
				With grd
					.AllowUserToResizeRows = False
					.AllowUserToResizeColumns = True
					.RowHeadersWidth = 25
					.SelectionMode = DataGridViewSelectionMode.FullRowSelect
					.ShowEditingIcon = False
					.RowHeadersVisible = False
					.BackgroundColor = SystemColors.Window
					'Designer-Aufgaben-Einstellung
					.AllowUserToAddRows = False
					.ReadOnly = True
					.AllowUserToDeleteRows = False
					.AllowUserToOrderColumns = False
				End With
				AddHandler grd.DataError, AddressOf DgvErrorHandling
			End If
		Next
	End Sub

#End Region 'DGV

End Class
