﻿Imports System.IO, System.Runtime.CompilerServices, System.Drawing

Public Module Files

    Public Function SetFileName(FileExtensionWithDot As String, Name As String) As String
        If FileExtensionWithDot = "" Then Return "FEHLER! Keine Dateiendung!"
        Return $"{Name}{FileExtensionWithDot}"
    End Function

    Public Function SetFileName(FileExtensionWithDot As String, ParamArray NamePart() As String) As String
        If FileExtensionWithDot = "" Then Return "FEHLER! Keine Dateiendung!"
        Dim FileName = ""
        For Each part In NamePart
            If part = "" Then Continue For
            If FileName = "" Then
                FileName = part
            Else
                FileName = $"{FileName}_{part}"
            End If
        Next
        FileName = $"{FileName}{FileExtensionWithDot}"
        Return FileName
    End Function

    <Extension>
    Public Function IsValidFileName(name As String) As Boolean
        If name Is Nothing OrElse name = "" Then
            msgExclamation("kein Name angegeben!")
            Return False
        End If
        For Each badChar As Char In Path.GetInvalidFileNameChars
            If name.Contains(badChar) Then
                msgExclamation($"""{badChar}"" darf nicht in einem Dateinamen vorkommen!")
                Return False
            End If
        Next
        Return True
    End Function

    <Extension>
    Public Function IsValidFolderName(name As String) As Boolean
        If name Is Nothing OrElse name = "" Then
            msgExclamation("kein Name angegeben!")
            Return False
        End If
        For Each badChar As Char In Path.GetInvalidPathChars
            If name.Contains(badChar) Then
                msgExclamation($"""{badChar}"" darf nicht in einem Ordnenamen vorkommen!")
                Return False
            End If
        Next
        Return True
    End Function

    ''' <summary> Überprüft, ob eine Datei bereits geöffnet ist.</summary>
    <Extension>
    Public Function IsInUse(fileName As String) As Boolean
        If Not File.Exists(fileName) Then Return False
        Try
            Using fs = File.OpenWrite(fileName)
            End Using
            Return False
        Catch
            Return True
        End Try
    End Function

    ''' <summary> Returned einen ausgewählten Ordnerpfad </summary>
    Public Function getPath(Optional StartSuchOrdner As String = "") As String
        Using fldDlg As New FolderBrowserDialog
            If StartSuchOrdner <> "" Then
                fldDlg.SelectedPath = StartSuchOrdner
            End If
            If fldDlg.ShowDialog = DialogResult.OK Then
                Return fldDlg.SelectedPath
            Else
                msgError("keine Angabe!")
                Return ""
            End If
        End Using
    End Function

    <Extension>
    Public Sub getPathToTextBox(tb As TextBox)
        Using fldDlg As New FolderBrowserDialog
            If fldDlg.ShowDialog = DialogResult.OK Then
                tb.Text = fldDlg.SelectedPath
            Else
                tb.Text = "keine Angabe!"
            End If
        End Using
    End Sub

    Public Sub checkDir(tb As TextBox)
        If Not Directory.Exists(tb.Text) Then
            tb.BackColor = Color.Red
        Else
            tb.BackColor = SystemColors.Control
        End If
    End Sub

    Public Function PathExists(Path As String) As Boolean
        Return Directory.Exists(Path)
    End Function

    ''' <summary> Speichern unter... -Dialog </summary>
    Public Function saveAs(Filter As String) As String
        Using fiDlg As New SaveFileDialog
            fiDlg.Filter = Filter
            If fiDlg.ShowDialog = DialogResult.OK Then
                Return fiDlg.FileName
            Else
                msgError("keine Eingabe!")
                Return ""
            End If
        End Using
    End Function

    Public Function saveAsWithInitialDirAndFilename(Dir As String, Filename As String, Filter As String) As String
        Using fiDlg As New SaveFileDialog
            fiDlg.Filter = Filter
            fiDlg.InitialDirectory = Dir
            fiDlg.FileName = Filename
            If fiDlg.ShowDialog = DialogResult.OK Then
                Return fiDlg.FileName
            Else
                msgError("keine Eingabe!")
                Return ""
            End If
        End Using
    End Function

    ''' <summary>OpenFileDialog optional mit Filter auf Dateiendungen und Startverzeichnis</summary>
    ''' <returns>Dateiname inkl. Pfad</returns>
    Public Function GetFileNameWithPath(Optional Filter As String = "", Optional InitialDir As String = "") As String
        Using fiDlg As New OpenFileDialog
            If Filter <> "" Then fiDlg.Filter = Filter
            If InitialDir <> "" Then fiDlg.InitialDirectory = InitialDir
            If fiDlg.ShowDialog = DialogResult.OK Then
                Return fiDlg.FileName
            Else
                msgError("Es wurde keine Datei ausgewählt!")
                Return ""
            End If
        End Using
    End Function

    ''' <summary>OpenFileDialog optional mit Filter auf Dateiendungen und Startverzeichnis</summary>
    ''' <returns>Dateiname ohne Pfad</returns>
    Public Function GetFileNameWithoutPath(Optional Filter As String = "", Optional InitialDir As String = "") As String
        Using fiDlg As New OpenFileDialog
            If Filter <> "" Then fiDlg.Filter = Filter
            If InitialDir <> "" Then fiDlg.InitialDirectory = InitialDir
            If fiDlg.ShowDialog = DialogResult.OK Then
                Return fiDlg.SafeFileName
            Else
                msgError("Es wurde keine Datei ausgewählt!")
                Return ""
            End If
        End Using
    End Function

    ''' <summary>OpenFileDialog zur Auswahl mehrerer Dateien. Optional mit Filter und Startverzeichnis</summary>
    ''' <returns>Array aus mehreren Dateinamen inkl. Pfad</returns>
    Public Function GetMultipleFileNamesWithPath(Optional Filter As String = "", Optional InitialDir As String = "") As String()
        Using fiDlg As New OpenFileDialog
            If Filter <> "" Then fiDlg.Filter = Filter
            If InitialDir <> "" Then fiDlg.InitialDirectory = InitialDir
            fiDlg.Multiselect = True
            If fiDlg.ShowDialog = DialogResult.OK Then
                Return fiDlg.FileNames
            Else
                msgError("Es wurde keine Datei ausgewählt!")
                Return Nothing
            End If
        End Using
    End Function

    Public Function moveFile(selectedFileAndPath As String, moveTo As String) As Boolean
        If selectedFileAndPath Is Nothing Then Return False
        If moveTo Is Nothing Then Return False
        Try
            File.Move(selectedFileAndPath, moveTo)
            Return True
        Catch ex As Exception
            msgEx(ex)
            Return False
        End Try
    End Function

    Public Function copyFile(selectedFileAndPath As String, moveTo As String) As Boolean
        If selectedFileAndPath Is Nothing Then Return False
        If moveTo Is Nothing Then Return False
        Try
            File.Copy(selectedFileAndPath, moveTo)
            Return True
        Catch ex As Exception
            msgEx(ex)
            Return False
        End Try
    End Function

End Module
