﻿''' <summary>
''' Provides an ini text reader which can be used when reading data from a string.
''' </summary>
Public Class IniStringReader
    Inherits IniTextReader

    Private strStringValue As String
    Private intOffset As Integer

    ''' <summary>
    ''' Initializes a new instance of the IniLib.IniStringReader class.
    ''' </summary>
    ''' <param name="stringValue">A string from which to read the content of the ini file.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal stringValue As String)
        strStringValue = stringValue
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.IniStringReader class.
    ''' </summary>
    ''' <param name="stringValue">A string from which to read the content of the ini file.</param>
    ''' <param name="stringBufferDefaultLength">Specifies the default length of the string buffer used in general string operations.</param>
    Public Sub New(ByVal stringValue As String, ByVal stringBufferDefaultLength As Integer)
        MyBase.New(stringBufferDefaultLength)
        strStringValue = stringValue
    End Sub

    ''' <summary>
    ''' Returns the current character.
    ''' </summary>
    Public Overrides ReadOnly Property CurrentChar As Char
        Get
            Return strStringValue(intOffset)
        End Get
    End Property

    ''' <summary>
    ''' Retrieves whether the end of content is reached.
    ''' </summary>
    Public Overrides ReadOnly Property EndOfContent As Boolean
        Get
            Return intOffset >= strStringValue.Length - 1
        End Get
    End Property

    ''' <summary>
    ''' Reads characters into the specified buffer starting at the specified index. Returns the amount of characters read.
    ''' </summary>
    ''' <param name="chars">The buffer to fill with characters.</param>
    ''' <param name="index">The index at which to start filling the buffer.</param>
    ''' <param name="count">The maximum amount of characters to read.</param>
    ''' <returns>Returns the amount of characters read into the buffer.</returns>
    Public Overrides Function ReadChars(ByVal chars() As Char, ByVal index As Integer, ByVal count As Integer) As Integer
        ReadChars = Math.Min(count, strStringValue.Length - intOffset)
        strStringValue.CopyTo(intOffset, chars, index, ReadChars)
    End Function

    ''' <summary>
    ''' Reads all characters in the stream starting from the current offset.
    ''' </summary>
    Public Overrides Function ReadToEnd() As String
        ReadToEnd = strStringValue.Substring(intOffset)
        intOffset = strStringValue.Length
    End Function

    ''' <summary>
    ''' Moves on to the next character.
    ''' </summary>
    Public Overrides Sub MoveNext()
        If Not EndOfContent Then
            intOffset += 1
        Else
            Throw New ArgumentException("End of content reached.")
        End If
    End Sub
End Class
