﻿Imports System.Text, Microsoft.VisualBasic

''' <summary>
''' Provides general operations when reading strings from streams while using a buffer.
''' </summary>
Public Class TextStreamReader
    Inherits TextReader

    Private lngPosition As Long
    Private strmUnderlyingStream As IO.Stream
    Private encEncoding As System.Text.Encoding
    Private intBufferedBytes As Integer
    Private intBufferPosition As Integer
    Private intUntranslatedBytes As Integer
    Private btByteBuffer() As Byte
    Private chrCharBuffer() As Char
    Private intBufferedChars As Integer

    ''' <summary>
    ''' Initializes a new instance of the IniLib.TextStreamReader class using the specified stream as the data source.
    ''' </summary>
    ''' <param name="underlyingStream">The stream from which to read.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream)
        strmUnderlyingStream = underlyingStream
        lngPosition = underlyingStream.Position
        btByteBuffer = New Byte(32767) {}
        chrCharBuffer = New Char(Encoding.Default.GetMaxCharCount(btByteBuffer.Length) - 1) {}
        encEncoding = Text.Encoding.Default
        Buffer()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.TextStreamReader class using the specified stream as the data source and buffering the data into a byte array with the specified size.
    ''' </summary>
    ''' <param name="underlyingStream">The stream from which to read.</param>
    ''' <param name="byteBufferSize">The size of the byte buffer used for reading operations.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal byteBufferSize As Integer)
        strmUnderlyingStream = underlyingStream
        btByteBuffer = New Byte(byteBufferSize - 1) {}
        chrCharBuffer = New Char(Encoding.Default.GetMaxCharCount(byteBufferSize) - 1) {}
        encEncoding = Text.Encoding.Default
        Buffer()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.TextStreamReader class using the specified stream as the data source and buffering the data into a byte array with the specified size.
    ''' </summary>
    ''' <param name="underlyingStream">The stream from which to read.</param>
    ''' <param name="byteBufferSize">The size of the byte buffer used for reading operations.</param>
    ''' <param name="stringBufferDefaultLength">The default length for string builders used for general string reading operations.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal byteBufferSize As Integer, ByVal stringBufferDefaultLength As Integer)
        MyBase.New(stringBufferDefaultLength)
        lngPosition = underlyingStream.Position
        strmUnderlyingStream = underlyingStream
        btByteBuffer = New Byte(byteBufferSize - 1) {}
        chrCharBuffer = New Char(Encoding.Default.GetMaxCharCount(byteBufferSize) - 1) {}
        encEncoding = Text.Encoding.Default
        Buffer()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.TextStreamReader class using the specified stream as the data source encoded as specified and buffering the data into a byte array with the specified size.
    ''' </summary>
    ''' <param name="underlyingStream">The stream from which to read.</param>
    ''' <param name="encoding">The encoding of the data read from the stream.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal encoding As Text.Encoding)
        strmUnderlyingStream = underlyingStream
        lngPosition = underlyingStream.Position
        btByteBuffer = New Byte(32767) {}
        chrCharBuffer = New Char(encoding.GetMaxCharCount(btByteBuffer.Length) - 1) {}
        encEncoding = encoding
        Buffer()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.TextStreamReader class using the specified stream as the data source encoded as specified and buffering the data into a byte array with the specified size.
    ''' </summary>
    ''' <param name="underlyingStream">The stream from which to read.</param>
    ''' <param name="encoding">The encoding of the data read from the stream.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal encoding As Text.Encoding, ByVal byteBufferSize As Integer)
        strmUnderlyingStream = underlyingStream
        lngPosition = underlyingStream.Position
        btByteBuffer = New Byte(byteBufferSize - 1) {}
        chrCharBuffer = New Char(encoding.GetMaxCharCount(byteBufferSize) - 1) {}
        encEncoding = encoding
        Buffer()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.TextStreamReader class reading from text encoded in the specified encoding from the specified stream while using a byte buffer whose size is as specified.
    ''' When operating on strings the default buffer length for string builders is used for general operations.
    ''' </summary>
    ''' <param name="underlyingStream">Specifies the stream from which to read.</param>
    ''' <param name="encoding">The encoding of the string data stored in the stream.</param>
    ''' <param name="byteBufferSize">Specifies the byte buffer size to use when buffering data from the stream.</param>
    ''' <param name="stringBufferDefaultLength">The string builder default length to use for general read operations on strings.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal encoding As Text.Encoding, ByVal byteBufferSize As Integer, ByVal stringBufferDefaultLength As Integer)
        MyBase.New(stringBufferDefaultLength)
        lngPosition = underlyingStream.Position
        strmUnderlyingStream = underlyingStream
        btByteBuffer = New Byte(byteBufferSize - 1) {}
        chrCharBuffer = New Char(encoding.GetMaxCharCount(byteBufferSize) - 1) {}
        encEncoding = encoding
        Buffer()
    End Sub

   ''' <summary>
    ''' Buffers the following content while keeing unread data. Returns whether the amount of buffered bytes is zero therefore the end of content is reached.
    ''' </summary>
    ''' <returns>Returns whether the amount of all buffered bytes including unused bytes is zero therefore the end of content is reached.</returns>
    Protected Function Buffer() As Boolean
        strmUnderlyingStream.Position = lngPosition
        If intUntranslatedBytes > 0 Then
            Array.Copy(btByteBuffer, btByteBuffer.Length - intUntranslatedBytes - 1, btByteBuffer, 0, intUntranslatedBytes)
        End If
        intBufferedBytes = strmUnderlyingStream.Read(btByteBuffer, intUntranslatedBytes, Math.Min(btByteBuffer.Length, encEncoding.GetMaxByteCount(chrCharBuffer.Length)) - intUntranslatedBytes)
        intBufferedChars = encEncoding.GetChars(btByteBuffer, 0, intBufferedBytes, chrCharBuffer, 0)
        intBufferPosition = 0
        lngPosition += intBufferedBytes
        intUntranslatedBytes = btByteBuffer.Length - encEncoding.GetByteCount(chrCharBuffer, 0, intBufferedChars)

        Return intBufferedBytes > 0
    End Function
    
    ''' <summary>
    ''' Returns the character currently selected.
    ''' </summary>
    Public Overrides ReadOnly Property CurrentChar As Char
        Get
            Return chrCharBuffer(intBufferPosition)
        End Get
    End Property
    
    ''' <summary>
    ''' Returns the characters decoded from the byte buffer.
    ''' </summary>
    ''' <value>The content of the char buffer should not be modified.</value>
    Protected ReadOnly Property CharBuffer() As Char()
        Get
            Return chrCharBuffer
        End Get
    End Property

    ''' <summary>
    ''' Returns the byte buffer used to decode characters from the text reader's underlying stream.
    ''' </summary>
    ''' <value>The content of the byte buffer should not be modified.</value>
    Protected ReadOnly Property ByteBuffer() As Byte()
        Get
            Return btByteBuffer
        End Get
    End Property

    ''' <summary>
    ''' Advances the buffer position by one and returns the character which is then at the buffer position.
    ''' </summary>
    Public Overrides Function ReadChar() As Char
        intBufferPosition += 1
        If intBufferPosition < intBufferedChars Then
            Return chrCharBuffer(intBufferPosition)
        ElseIf Buffer() Then
            Return chrCharBuffer(intBufferPosition)
        Else
            Throw New IO.EndOfStreamException()
        End If
    End Function

    ''' <summary>
    ''' Advances the buffer position depending on the content. View remarks for further information.
    ''' </summary>
    ''' <returns>Returns whether advancing was successful.</returns>
    ''' <remarks>As there are differences in treating line feed characters when it comes to different standards the IniLib supports these. The IniLib's rules of interpreting line feeds are as follows.
    ''' Cr, Lf, CrLf, LfCr are interpreted as one character (Cr = Carriage return, Lf = Line feed) while LfLf or CrCr are two characters.</remarks>
    Public Overrides Function TryReadChar(ByRef result As Char) As Boolean
        intBufferPosition += 1
        If intBufferPosition < intBufferedChars Then
            result = chrCharBuffer(intBufferPosition)
            Return True
        ElseIf Buffer() Then
            result = chrCharBuffer(intBufferPosition)
            Return True
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Reads the specified amount of bytes and returns them as an array.
    ''' </summary>
    ''' <param name="count">The amount of bytes to read.</param>
    Public Overridable Overloads Function ReadChars(ByVal count As Integer) As Char()
        Dim c(count - 1) As Char
        ReadChars(c, 0, count)
        Return c
    End Function

    ''' <summary>
    ''' Reads characters into the specified buffer and returns the total amount of characters read.
    ''' </summary>
    ''' <param name="buffer">The buffer to fill with characters.</param>
    Public Overridable Overloads Function ReadChars(ByVal buffer() As Char) As Integer
        Return ReadChars(buffer, 0, buffer.Length)
    End Function

    ''' <summary>
    ''' Reads characters into the specified buffer starting at the specified index. Returns the amount of characters read.
    ''' </summary>
    ''' <param name="buffer">The buffer to fill with characters.</param>
    ''' <param name="index">The index at which to start filling the buffer.</param>
    ''' <param name="count">The maximum amount of characters to read.</param>
    ''' <returns>Returns the amount of characters read into the buffer.</returns>
    Public Overrides Function ReadChars(ByVal buffer() As Char, ByVal index As Integer, ByVal count As Integer) As Integer
        If count + intBufferPosition < intBufferedChars Then
            Array.Copy(chrCharBuffer, intBufferPosition, buffer, index, count)
            Return count
        Else
            Dim c As Integer = intBufferedChars - intBufferPosition
            Dim nc As Integer
            Array.Copy(chrCharBuffer, intBufferPosition, buffer, index, c)

            While c < count
                If Me.Buffer() Then
                    nc = Math.Min(count - c, intBufferedChars - intBufferPosition)
                    Array.Copy(buffer, intBufferPosition, buffer, index + c, nc)
                    intBufferPosition += nc
                Else
                    Return c
                End If
            End While
            Return c
        End If
    End Function

    ''' <summary>
    ''' Reads until the character passed to the specified match matches the requirements.
    ''' </summary>
    ''' <param name="match">The delegate that checks characters for matches.</param>
    Public Overrides Function ReadUntil(ByVal match As Predicate(Of Char)) As String
        Dim startIndex As Integer = intBufferPosition
        Dim sb As New StringBuilder(StringBufferDefaultLength)

        While Not match(chrCharBuffer(intBufferPosition))
            If intBufferPosition < intBufferedChars Then
                sb.Append(chrCharBuffer(intBufferPosition))
                intBufferPosition += 1
            ElseIf Not Buffer() Then
                Return sb.ToString
            End If

        End While
        Return sb.ToString
    End Function

    ''' <summary>
    ''' Skips all space and tab characters.
    ''' </summary>
    Public Overrides Sub SkipSpaces()
        If (chrCharBuffer(intBufferPosition) = " "c OrElse chrCharBuffer(intBufferPosition) = Chr(9)) Then
            intBufferPosition += 1
            While ((intBufferPosition + 1 < intBufferedChars OrElse Buffer()) AndAlso (chrCharBuffer(intBufferPosition + 1) = " "c OrElse chrCharBuffer(intBufferPosition + 1) = Chr(9)))
                intBufferPosition += 1
            End While
        End If
    End Sub

    ''' <summary>
    ''' Reads a word.
    ''' </summary>
    Public Overrides Function ReadWord() As String
        Dim sb As New StringBuilder(StringBufferDefaultLength)
        While Not Char.IsWhiteSpace(chrCharBuffer(intBufferPosition))
            If intBufferPosition < intBufferedChars Then
                sb.Append(chrCharBuffer(intBufferPosition))
                intBufferPosition += 1
            ElseIf Not Buffer() Then
                Return sb.ToString
            End If
        End While
        Return sb.ToString
    End Function

    ''' <summary>
    ''' Reads a string with the specified length.
    ''' </summary>
    ''' <param name="length">The amount of characters to read.</param>
    Public Overrides Function ReadString(ByVal length As Integer) As String
        Dim sb As New StringBuilder(length)

        While (sb.Length < length)
            If intBufferPosition < intBufferedChars Then
                sb.Append(chrCharBuffer(intBufferPosition))
                intBufferPosition += 1
            ElseIf Not Buffer() Then
                Return sb.ToString
            End If

        End While

        Return sb.ToString()
    End Function

    ''' <summary>
    ''' Reads a line.
    ''' </summary>
    Public Overrides Function ReadLine() As String
        Dim sb As New StringBuilder(StringBufferDefaultLength)

        While Not IsLinefeedChar(chrCharBuffer(intBufferPosition))
            If intBufferPosition < intBufferedChars Then
                sb.Append(chrCharBuffer(intBufferPosition))
                intBufferPosition += 1
            ElseIf Not Buffer() Then
                Return sb.ToString()
            End If
        End While
        If (intBufferPosition < intBufferedChars) OrElse Buffer() Then
            SkipLinefeed()
        End If
        Return sb.ToString
    End Function


    ''' <summary>
    ''' Skips characters while the match parameter indicates that they match the requirements.
    ''' </summary>
    ''' <param name="match">A delegate specifying the requirements of the characters.</param>
    Public Overrides Sub SkipWhile(ByVal match As Predicate(Of Char))
        If match(chrCharBuffer(intBufferPosition)) Then
            While ((intBufferPosition + 1 < intBufferedChars OrElse Buffer()) AndAlso match(chrCharBuffer(intBufferPosition + 1)))
                intBufferPosition += 1
            End While
        End If
    End Sub

    ''' <summary>
    ''' Gets the stream from which data is read.
    ''' </summary>
    Public ReadOnly Property UnderlyingStream As IO.Stream
        Get
            Return strmUnderlyingStream
        End Get
    End Property

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="offset"></param>
    ''' <remarks></remarks>
    Public Sub Seek(ByVal offset As Long)
        If offset >= 0 AndAlso offset < strmUnderlyingStream.Length Then
            lngPosition = offset
        Else
            Throw New ArgumentOutOfRangeException("offset")
        End If
    End Sub

    ''' <summary>
    ''' Sets the position of this IniLib.IniStreamReader instance to the specified offset.
    ''' </summary>
    ''' <param name="offset">The offset from the stream's start in bytes.</param>
    Public Sub Seek(ByVal offset As Long, ByVal origin As IO.SeekOrigin)
        Select Case origin
            Case IO.SeekOrigin.Begin
                If offset >= 0 AndAlso offset < strmUnderlyingStream.Length Then
                    lngPosition = offset
                Else
                    Throw New ArgumentOutOfRangeException("offset")
                End If

                Exit Select
            Case IO.SeekOrigin.Current
                If offset + strmUnderlyingStream.Position >= 0 AndAlso offset + strmUnderlyingStream.Position < strmUnderlyingStream.Length Then
                    lngPosition += offset
                Else
                    Throw New ArgumentOutOfRangeException("offset")
                End If

                Exit Select
            Case IO.SeekOrigin.End
                If offset >= 0 AndAlso offset < strmUnderlyingStream.Length Then
                    lngPosition = strmUnderlyingStream.Length - offset - 1
                Else
                    Throw New ArgumentOutOfRangeException("offset")
                End If

                Exit Select
        End Select
        Buffer()
    End Sub

    ''' <summary>
    ''' Sets the position of the stream to its actual position regarding the character buffer's offset.
    ''' </summary>
    Public Overrides Sub Flush()
        strmUnderlyingStream.Position = lngPosition - intBufferedBytes + encEncoding.GetByteCount(chrCharBuffer, 0, intBufferedBytes)
    End Sub

    ''' <summary>
    ''' Retrieves whether the end of content is reached.
    ''' </summary>
    Public Overrides ReadOnly Property EndOfContent As Boolean
        Get
            Return intBufferPosition >= intBufferedChars - 1 AndAlso lngPosition >= strmUnderlyingStream.Length - 1
        End Get
    End Property

    ''' <summary>
    ''' Reads all characters in the stream starting from the current offset.
    ''' </summary>
    Public Overrides Function ReadToEnd() As String
        Dim sb As New StringBuilder(StringBufferDefaultLength) 'TODO: optimize length
        sb.Append(chrCharBuffer, intBufferPosition, intBufferedChars - intBufferPosition)
        If Not EndOfContent Then
            While Buffer()
                sb.Append(chrCharBuffer, 0, intBufferedChars)
            End While
        End If
        Return sb.ToString
    End Function

    ''' <summary>
    ''' Advances the current reader position by one.
    ''' </summary>
    Public Overrides Sub MoveNext()
        intBufferPosition += 1
        If intBufferPosition = intBufferedChars AndAlso Not Buffer() Then
            Throw New IO.EndOfStreamException()
        End If
    End Sub
End Class
