﻿Imports System.Data.OleDb, System.Runtime.CompilerServices, System.Diagnostics, System.IO
Imports OfficeOpenXml, Microsoft.Office.Interop

Public Module Office

#Region "Excel"

    <Extension>
    Public Function PasteClipBoardToTempTable(dgv As DataGridView) As DataTable
        Dim dt = New DataTable
        For Each col In dgv.ColumnsX
            dt.Columns.Add(col.HeaderText)
        Next
        Dim s = Clipboard.GetText()
        Dim lines = s.Split(Microsoft.VisualBasic.vbLf)
        If lines(0).Split(Microsoft.VisualBasic.vbTab).Length <> dgv.Columns.Count Then
            msgError("Die Anzahl der zu importierenden Spalten stimmt nicht überein!")
            Return Nothing
        End If
        For Each line In lines
            Dim itmArr = line.Split(Microsoft.VisualBasic.vbTab)
            If Not itmArr.IsEmpty Then dt.Rows.Add(itmArr)
        Next
        'showDebugDGV(dt)
        Return dt
    End Function

    ''' <summary>Splittet eine vorhandene Excel-Datei anhand der Datum-Spalte nach Monat auf und speichert neue Dateien pro Monat inkl. Header</summary>
    Public Sub SplitExcelDateiProMonat()
        If msgOkCancel("im Folgenden bitte die Excel-Datei auswählen", "Excel-Import") = DialogResult.OK Then
            Dim importFile = GetFileNameWithPath("Exce-Dateien (*.xlsx)|*.xlsx")
            If importFile = "" Then Return
            Dim dtExcel = ExcelImportToTempDataTable(importFile, True) 'Sheet wird hier mit ausgewählt
            If dtExcel Is Nothing Then Return
            Dim ColDatum = ""
            Using dlg As New dlgListboxAuswahl
                dlg.Text = "Datum-Spalte auswählen"
                For Each col In dtExcel.ColumnsX
                    dlg.lbAuswahl.Items.Add(col.ColumnName)
                Next
                If dlg.ShowDialog = DialogResult.OK Then
                    ColDatum = dlg.lbAuswahl.SelectedItem.ToString
                Else
                    Return
                End If
            End Using
            If Not dtExcel.colExists(ColDatum) Then
                msgExclamation($"Die Datumsspalte {ColDatum} existiert nicht!")
                Return
            End If
            msgInformation("Bitte im folgenden Dialog ein Ausgabeverzeichnis für die gesplitteten Dateien auswählen.")
            Dim Ausgabeverzeichnis = getPath(importFile.LastLeftCut("\"c))
            Dim DateiPrefix = ""
            Using dlg As New dlgTextInput
                dlg.Text = "Dateiprefix für Ausgabedateien"
                If dlg.ShowDialog = DialogResult.OK Then
                    DateiPrefix = dlg.tbInput.Text
                    If Not DateiPrefix.IsValidFileName Then
                        Return
                    End If
                Else
                    Return
                End If
            End Using
            Dim FileCount = 0
            Using dlgIB As New dlgIsBusy("Bitte warten...", "Dateien werden erzeugt...")
                Using New Threading.WaitDlg(dlgIB)
                    For i = 1 To 12
                        Dim MonatNr = i
                        dlgIB.UpdateDescription($"{MonatName(MonatNr)}")
                        Using dtmonat As New DataTable
                            For Each col In dtExcel.ColumnsX
                                dtmonat.Columns.Add(col.ColumnName)
                            Next
                            For Each rw In dtExcel.RowsX
                                If Not rw.Item(ColDatum).ToString = "" Then
                                    If ExcelDateOADate(rw.Item(ColDatum).ToString).Month = MonatNr Then
                                        dtmonat.Rows.Add(rw.ItemArray)
                                    End If
                                End If
                            Next
                            If dtmonat.Rows.Count > 0 Then
                                dtmonat.ExportDtToExcel(MonatName(MonatNr), $"{Ausgabeverzeichnis}\{DateiPrefix}-{MonatNr}.xlsx")
                                FileCount += 1
                            End If
                        End Using
                    Next
                End Using
            End Using
            msgInformation($"Abgeschlossen, es wurden {FileCount} Dateien in {Ausgabeverzeichnis} erzeugt.")
        End If
    End Sub

#Region "Worksheet-Settings (EPPlus)"

    ''' <summary> zeichnet um jede beschriebene Zelle einen kompletten Rahmen </summary>
    <Extension>
    Private Sub SetBordersAllAround(wks As ExcelWorksheet)
        With wks.Cells(wks.Dimension.Address).Style.Border
            .Top.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
            .Left.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
            .Right.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
            .Bottom.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
        End With
    End Sub

#End Region 'Worksheet-Settings

#Region "Export DGV(s) -> Excel"

#Region "Export mit ungebundenen Werten"

    <Extension>
    Public Function GetDgvColsForExportTuple(dgv As DataGridView, excel As Boolean) As (dgvCols As List(Of dgvCol), OnlyMarkedRows As Boolean, UseHeader As Boolean, Excel As Boolean)
        Dim lstCol As New List(Of dgvCol)
        Using dlg As New dlgDGVcopyExport
            Dim listbox = dlg.chkLbSpalten
            For Each col In dgv.ColumnsX
                If col.Visible Then lstCol.Add(New dgvCol With {.name = col.Name, .headertext = convertTrimString(col.HeaderText), .valueType = col.ValueType})
            Next
            dlg.Text = If(excel, "Export Excel", "in Zwischenablage kopieren")
            listbox.DataSource = lstCol
            listbox.DisplayMember = "headertext"
            listbox.ValueMember = "name"
            For i = 0 To listbox.Items.Count - 1
                listbox.SetItemChecked(i, True)
            Next
            If dlg.ShowDialog <> DialogResult.OK Then Return Nothing
            Dim selectedCols As New List(Of dgvCol)
            For Each chkItm In listbox.CheckedItems
                selectedCols.Add(DirectCast(chkItm, dgvCol))
            Next
            Return (selectedCols, dlg._Selection = "marked", dlg._header, excel)
        End Using
    End Function

    ''' <summary>Erzeugt eine neue Datatable, vorgefüllt mit ausgewählten Spalten, Zeilen und Datengebundenen Items. Ungebundene Werte müssen im Nachgang hinzugefügt werden</summary>
    Public Function GetDataTableWithoutUnboundValues(dgv As DataGridView, excel As Boolean) As DataTable
        Dim tpl = dgv.GetDgvColsForExportTuple(excel)
        Dim dtExport = dgv.NewDataTableWithColumnsFromDgv
        If tpl.OnlyMarkedRows Then
            For Each row In dgv.SelectedRowsX
                AddRowWithValues(dtExport, row, dgv)
            Next
        Else
            For Each row In dgv.RowsX
                AddRowWithValues(dtExport, row, dgv)
            Next
        End If
        For Each col In dtExport.ColumnsX
            If Not tpl.dgvCols.Select(Function(x) x.name).Contains(col.ColumnName) Then dtExport.Columns.Remove(col)
        Next
        _UseHeader = tpl.UseHeader
        Return dtExport
    End Function

    Private Sub AddRowWithValues(dtExport As DataTable, row As DataGridViewRow, dgv As DataGridView)
        dtExport.Rows.Add()
        For Each col In dgv.ColumnsX
            dtExport.Rows(dtExport.Rows.Count - 1).Item(col.Name) = row.Cells(col.Name).FormattedValue
        Next
    End Sub

    <Extension>
    Public Sub CopyOrExportDataTable(dtExport As DataTable, Excel As Boolean, Optional worksheetname As String = "Export")
        SetCellValueForExport(dtExport)
        If Excel Then
            ExportDtToExcel(dtExport, worksheetname)
        Else
            Clipboard.SetDataObject(setClipboardContent(dtExport, _UseHeader), True)
        End If
    End Sub

    Private Sub SetCellValueForExport(dtExport As DataTable)
        For Each row In dtExport.RowsX
            For Each col In dtExport.ColumnsX
                Dim colstr = row.Item(col).ToString
                If colstr = "False" Then row.Item(col) = "Nein"
                If colstr = "True" Then row.Item(col) = "Ja"
            Next
        Next
    End Sub

#End Region 'Export mit ungebundenen Werten

    ''' <summary>Exportiert den Inhalt mehrerer DGVs inkl. Angabe von Sheetname in eine Excel-Datei via EPPlus</summary>
    Public Sub ExportDgvsToExcel(DI As Dictionary(Of DataGridView, String))
        Dim excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        If excelFile = "" Then Return
        If File.Exists(excelFile) Then File.Delete(excelFile)
        For Each entry In DI
            Dim dgv = entry.Key
            Dim wsn = entry.Value
            ExportDgvWsnToExcel(dgv, wsn, excelFile)
        Next
        If msgQuestionInfo($"Der Export wurde unter {excelFile} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
            Process.Start(excelFile)
        End If
    End Sub

    ''' <summary>Exportiert den Inhalt eines DGVs inkl. Angabe von Sheetname in eine Excel-Datei via EPPlus</summary>
    <Extension>
    Public Sub ExportDgvToExcel(dgv As DataGridView, worksheet As String)
        Dim excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        If excelFile = "" Then Return
        If File.Exists(excelFile) Then File.Delete(excelFile)
        ExportDgvWsnToExcel(dgv, worksheet, excelFile)
        If msgQuestionInfo($"Der Export wurde unter {excelFile} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
            Process.Start(excelFile)
        End If
    End Sub

    Private Sub ExportDgvWsnToExcel(dgv As DataGridView, wsn As String, excelFile As String)
        Dim eFi = New FileInfo(excelFile)
        Using pck As New ExcelPackage(eFi)
            Dim wks = pck.Workbook.Worksheets.Add(wsn)
            Dim dtExport As New DataTable
            Dim visibleColNames = dgv.Columns.Cast(Of DataGridViewColumn).Where(Function(x) x.Visible).Select(Function(x) x.HeaderText).ToList
            visibleColNames.ForEach(AddressOf dtExport.Columns.Add)
            For Each rw As DataGridViewRow In dgv.Rows
                dtExport.Rows.Add()
                For Each cell As DataGridViewCell In rw.Cells
                    If cell.OwningColumn.Visible = True Then
                        Dim colHeader = cell.OwningColumn.HeaderText
                        dtExport.Rows(dtExport.Rows.Count - 1)(colHeader) = CStr(cell.FormattedValue)
                    End If
                Next
            Next
            wks.Cells("A1").LoadFromDataTable(dtExport, True)
            wks.SetBordersAllAround
            wks.Cells(wks.Dimension.Address).AutoFitColumns()
            pck.Save()
        End Using
    End Sub

#Region "Selection DGV"

    Private _OnlyMarkedRows As Boolean
    Private _UseHeader As Boolean
    <Extension>
    Public Function GetDgvColsForExport(dgv As DataGridView, excel As Boolean) As List(Of dgvCol)
        _OnlyMarkedRows = False
        _UseHeader = True
        Dim lstCol As New List(Of dgvCol)
        Using dlg As New dlgDGVcopyExport
            Dim listbox = dlg.chkLbSpalten
            For Each col In dgv.ColumnsX
                If col.Visible Then lstCol.Add(New dgvCol With {.name = col.Name, .headertext = convertTrimString(col.HeaderText), .valueType = col.ValueType})
            Next
            dlg.Text = If(excel, "Export Excel", "in Zwischenablage kopieren")
            listbox.DataSource = lstCol
            listbox.DisplayMember = "headertext"
            listbox.ValueMember = "name"
            For i = 0 To listbox.Items.Count - 1
                listbox.SetItemChecked(i, True)
            Next
            If dlg.ShowDialog <> DialogResult.OK Then Return Nothing
            Dim selectedCols As New List(Of dgvCol)
            For Each chkItm In listbox.CheckedItems
                selectedCols.Add(DirectCast(chkItm, dgvCol))
            Next
            _OnlyMarkedRows = dlg._Selection = "marked"
            _UseHeader = dlg._header
            Return selectedCols
        End Using
    End Function

    <Extension>
    Public Sub CopyOrExportSelection(dgv As DataGridView, excel As Boolean, Optional e As KeyEventArgs = Nothing)
        If e IsNot Nothing Then e.SuppressKeyPress = True
        Dim selectedCols = dgv.GetDgvColsForExport(excel)
        Select Case _OnlyMarkedRows
            Case False
                If excel Then
                    exportSelectionToExcel(dgv, "Export", selectedCols, _UseHeader)
                Else
                    copySelectionToClipboard(dgv, selectedCols, _UseHeader)
                End If
            Case True
                Dim lstRow = dgv.SelectedRowsX.OrderBy(Function(rw) rw.Index).ToList '(From rw In dgv.SelectedRows.Cast(Of DataGridViewRow)() Order By rw.Index).ToList
                If excel Then
                    exportSelectionToExcel(dgv, "Export", selectedCols, _UseHeader, lstRow)
                Else
                    WaitCursor()
                    copySelectionToClipboard(dgv, selectedCols, _UseHeader, lstRow)
                    DefaultCursor()
                End If
        End Select
    End Sub

    ''' <summary>kopiert den selektierten Inhalt eines DGVs in die Zwischenablage</summary>
    Private Sub copySelectionToClipboard(dgv As DataGridView, selectedCols As List(Of dgvCol), Header As Boolean, Optional lstRow As List(Of DataGridViewRow) = Nothing)
        If selectedCols Is Nothing Then Return
        Dim dtExport As New DataTable
        For Each col In selectedCols
            Dim dtCol As New DataColumn With {.ColumnName = col.name, .Caption = col.headertext}
            dtExport.Columns.Add(dtCol)
        Next
        SetCellValueForExport(dtExport, dgv, lstRow)
        Clipboard.SetDataObject(setClipboardContent(dtExport, Header), True)
    End Sub

    ''' <summary> wandelt den Inhalt einer DateTable in das tabellarische Zwischenablage-Format um </summary>
    Private Function setClipboardContent(dt As DataTable, header As Boolean) As String
        Dim tab = Convert.ToChar(Keys.Tab)
        Dim colString = ""
        Dim rowString = ""
        If dt.Rows.Count > 0 Then
            'Header
            If header Then
                colString = ""
                For i = 0 To dt.Columns.Count - 1
                    If i = 0 Then
                        colString = $"{dt.Columns(i).Caption}"
                    Else
                        colString = $"{colString}{tab}{dt.Columns(i).Caption}"
                    End If
                Next
            End If
            'Rows
            Dim lstRowString As New List(Of String)
            For Each rw As DataRow In dt.Rows
                rowString = ""
                For Each col In dt.ColumnsX
                    Dim itmString = convertTrimString(rw.Item(col).ToString)
                    If dt.Columns(0).ColumnName = col.ColumnName Then
                        rowString = $"{itmString}"
                    Else
                        rowString = $"{rowString}{tab}{itmString}"
                    End If
                Next
                lstRowString.Add(rowString)
            Next
            'Rows sauber anordnen
            Dim rwString = ""
            For i = 0 To lstRowString.Count - 1
                If i = 0 Then
                    rwString = $"{lstRowString(i)}{Environment.NewLine}"
                Else
                    rwString = $"{rwString}{lstRowString(i)}{Environment.NewLine}"
                End If
            Next
            'Ausgabe
            If header Then
                Return $"{colString}{Environment.NewLine}{rwString}"
            Else
                Return rwString
            End If
        Else
            Return ""
        End If
    End Function

    Public Class dgvCol
        Property name As String
        Property headertext As String
        Property valueType As Type
    End Class

    ''' <summary>Exportiert den selektierten Inhalt eines DGVs inkl. Angabe von Sheetname in eine Excel-Datei via EPPlus</summary>
    Private Sub exportSelectionToExcel(dgv As DataGridView, wsn As String, selectedCols As List(Of dgvCol), Header As Boolean, Optional lstrow As List(Of DataGridViewRow) = Nothing)
        Dim excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        If excelFile = "" Then Return
        If excelFile.IsInUse Then
            msgInformation($"Die Datei {excelFile} ist bereits geöffnet, bitte einen neuen Dateinamen angeben.")
            excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        Else
            If File.Exists(excelFile) Then File.Delete(excelFile)
        End If
        Dim eFi = New FileInfo(excelFile)
        Using pck As New ExcelPackage(eFi)
            Dim wks = pck.Workbook.Worksheets.Add(wsn)
            Dim dtExport As New DataTable
            For Each col In selectedCols
                Dim dtCol As New DataColumn
                With dtCol
                    .ColumnName = col.name
                    .Caption = col.headertext
                End With
                dtExport.Columns.Add(dtCol)
            Next
            SetCellValueForExport(dtExport, dgv, lstrow)
            wks.Cells("A1").LoadFromDataTable(dtExport, If(Header, True, False))
            With wks.Cells(wks.Dimension.Address).Style.Border
                .Top.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
                .Left.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
                .Right.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
                .Bottom.Style = OfficeOpenXml.Style.ExcelBorderStyle.Thin
            End With
            wks.Cells(wks.Dimension.Address).AutoFitColumns()
            pck.Save()
        End Using
        If msgQuestionInfo($"Der Export wurde unter {excelFile} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
            Process.Start(excelFile)
        End If
    End Sub

    Private Sub SetCellValueForExport(dtExport As DataTable, dgv As DataGridView, Optional lstRow As List(Of DataGridViewRow) = Nothing)
        Dim rowCollection = If(lstRow Is Nothing, dgv.RowsX, lstRow)
        For Each rw In rowCollection
            dtExport.Rows.Add()
            For Each cell In rw.CellsX
                If cell.OwningColumn.Visible = True Then
                    Dim colName = cell.OwningColumn.Name
                    Dim colValueType = cell.OwningColumn.ValueType
                    If colValueType = GetType(Boolean) Then
                        If colExist(dtExport, colName) Then dtExport.Rows(dtExport.Rows.Count - 1)(colName) = If(cell.FormattedValue.ToString = "False", "Nein", "Ja")
                    Else
                        If colExist(dtExport, colName) Then dtExport.Rows(dtExport.Rows.Count - 1)(colName) = cell.FormattedValue.ToString
                    End If
                End If
            Next
        Next
    End Sub

    Private Function colExist(dt As DataTable, colName As String) As Boolean
        Return dt.ColumnsX.Any(Function(x) x.ColumnName = colName)
    End Function

#End Region 'Selection DGV

#End Region 'Export DGV(s) -> Excel

#Region "Export DataTable(s) -> Excel"

    ''' <summary>Exportiert mehrere DataTables in eine Temporäre Excel-Datei und gibt diese zurück</summary>
    Public Function ExportDataTablesToOneExcelFileTemp(fi As FileInfo, dicDtAndWorksheetName As Dictionary(Of DataTable, String)) As FileInfo
        Dim excelFile = fi
        If excelFile.FullName.IsInUse Then
            msgExclamation($"Die Datei {excelFile} ist bereits geöffnet, abbruch!")
            Return Nothing
        Else
            If File.Exists(excelFile.FullName) Then File.Delete(excelFile.FullName)
        End If
        Dim eFi = New FileInfo(excelFile.FullName)
        Using pck As New ExcelPackage(eFi)
            For Each kvp In dicDtAndWorksheetName
                Dim wks = pck.Workbook.Worksheets.Add(kvp.Value)
                wks.Cells("A1").LoadFromDataTable(kvp.Key, True)
                wks.SetBordersAllAround
                wks.Cells(wks.Dimension.Address).AutoFitColumns()
            Next
            pck.Save()
        End Using
        Return excelFile
    End Function

    ''' <summary>Exportiert mehrere DataTables in eine Excel-Datei</summary>
    Public Sub ExportDataTablesToOneExcelFile(dicDtAndWorksheetName As Dictionary(Of DataTable, String))
        Dim excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        If excelFile = "" Then Return
        If excelFile.IsInUse Then
            msgInformation($"Die Datei {excelFile} ist bereits geöffnet, bitte einen neuen Dateinamen angeben.")
            excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        Else
            If File.Exists(excelFile) Then File.Delete(excelFile)
        End If
        Dim eFi = New FileInfo(excelFile)
        Using pck As New ExcelPackage(eFi)
            For Each kvp In dicDtAndWorksheetName
                Dim wks = pck.Workbook.Worksheets.Add(kvp.Value)
                wks.Cells("A1").LoadFromDataTable(kvp.Key, True)
                wks.SetBordersAllAround
                wks.Cells(wks.Dimension.Address).AutoFitColumns()
            Next
            pck.Save()
        End Using
        If msgQuestionInfo($"Der Export wurde unter {excelFile} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
            Process.Start(excelFile)
        End If
    End Sub

    ''' <summary>Exportiert eine komplette DataTables (egal, was das DGV anzeigt) in eine Excel-Datei via EPPlus</summary>
    <Extension>
    Public Sub ExportDtToExcel(dtExport As DataTable, worksheetname As String)
        Dim excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        If excelFile = "" Then Return
        If excelFile.IsInUse Then
            msgInformation($"Die Datei {excelFile} ist bereits geöffnet, bitte einen neuen Dateinamen angeben.")
            excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        Else
            If File.Exists(excelFile) Then File.Delete(excelFile)
        End If
        Dim eFi = New FileInfo(excelFile)
        Using pck As New ExcelPackage(eFi)
            Dim wks = pck.Workbook.Worksheets.Add(worksheetname)
            wks.Cells("A1").LoadFromDataTable(dtExport, If(_UseHeader, True, False))
            wks.SetBordersAllAround
            wks.Cells(wks.Dimension.Address).AutoFitColumns()
            'wks.PrinterSettings.Orientation = eOrientation.Landscape
            'wks.PrinterSettings.FitToPage = True
            'wks.Column(wks.Dimension.End.Column).PageBreak = True																	 
            pck.Save()
        End Using
        If msgQuestionInfo($"Der Export wurde unter {excelFile} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
            Process.Start(excelFile)
        End If
    End Sub

    ''' <summary>Exportiert eine komplette DataTables (egal, was das DGV anzeigt) in eine Excel-Datei via EPPlus</summary>
    <Extension>
    Public Sub ExportDtToExcel(dt As DataTable, worksheetname As String, pathandfilename As String, Optional OpenFileAfterExport As Boolean = False)
        If pathandfilename = "" Then Return
        If pathandfilename.IsInUse Then
            msgInformation($"Die Datei {pathandfilename} ist bereits geöffnet, bitte einen neuen Dateinamen angeben.")
            pathandfilename = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        Else
            If File.Exists(pathandfilename) Then File.Delete(pathandfilename)
        End If
        Dim eFi As New FileInfo(pathandfilename)
        Using pck As New ExcelPackage(eFi)
            Dim wks = pck.Workbook.Worksheets.Add(worksheetname)
            wks.Cells("A1").LoadFromDataTable(dt, True)
            wks.SetBordersAllAround
            wks.Cells(wks.Dimension.Address).AutoFitColumns()
            'wks.PrinterSettings.Orientation = eOrientation.Landscape
            'wks.PrinterSettings.FitToPage = True
            'wks.Column(wks.Dimension.End.Column).PageBreak = True
            pck.Save()
        End Using
        If OpenFileAfterExport Then
            If msgQuestionInfo($"Der Export wurde unter {pathandfilename} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
                Process.Start(pathandfilename)
            End If
        End If
    End Sub

    ''' <summary>Exportiert mehrere komplette DataTables (egal, was das DGV anzeigt) in eine Excel-Datei via EPPlus</summary>
    Public Sub ExportDtToExcel(DI As Dictionary(Of DataTable, String))
        Dim excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        If excelFile = "" Then Return
        If excelFile.IsInUse Then
            msgInformation($"Die Datei {excelFile} ist bereits geöffnet, bitte einen neuen Dateinamen angeben.")
            excelFile = saveAs("Excel-Dateien (*.xlsx)|*.xlsx")
        Else
            If File.Exists(excelFile) Then File.Delete(excelFile)
        End If
        Dim eFi As New FileInfo(excelFile)
        For Each entry In DI
            Dim dt = entry.Key
            Dim wsn = entry.Value
            Using pck As New ExcelPackage(eFi)
                Dim wks = pck.Workbook.Worksheets.Add(wsn)
                wks.Cells("A1").LoadFromDataTable(dt, True)
                wks.SetBordersAllAround
                wks.Cells(wks.Dimension.Address).AutoFitColumns()
                'wks.PrinterSettings.Orientation = eOrientation.Landscape
                'wks.PrinterSettings.FitToPage = True
                'wks.Column(wks.Dimension.End.Column).PageBreak = True
                pck.Save()
            End Using
        Next
        If msgQuestionInfo($"Der Export wurde unter {excelFile} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
            Process.Start(excelFile)
        End If
    End Sub

    ''' <summary>Exportiert eine komplette DataTable in eine temporäre Excel-Datei via EPPlus</summary>
    <Extension>
    Public Sub ExportDtToTempExcel(dt As DataTable, excelfile As String, wsn As String)
        Dim eFi As New FileInfo(excelfile)
        Using pck As New ExcelPackage(eFi)
            Dim wks = pck.Workbook.Worksheets.Add(wsn)
            wks.Cells("A1").LoadFromDataTable(dt, True)
            wks.SetBordersAllAround
            wks.Cells(wks.Dimension.Address).AutoFitColumns()
            pck.Save()
        End Using
    End Sub

#End Region 'Export DataTable(s) -> Excel

#Region "Import"

    ''' <summary> Erstellt eine Excel-Importvorlage mit angegebenen Spalten </summary>
    Public Sub CreateExcelImportVorlage(wsn As String, cols As List(Of String))
        Dim dt As New DataTable
        cols.ForEach(AddressOf dt.Columns.Add)
        dt.ExportDtToExcel(wsn)
    End Sub

    ''' <summary> Erstellt eine Excel-Importvorlage mit angegebenen Spalten </summary>
    Public Sub CreateExcelImportVorlage(wsn As String, ParamArray cols As String())
        Dim dt As New DataTable
        cols.ForEach(AddressOf dt.Columns.Add)
        dt.ExportDtToExcel(wsn)
    End Sub

    Private Function ConvertXlsToXlsx(file As String) As String
        Dim app As New Excel.Application
        app.Workbooks.Open(file).SaveAs(Filename:=file + "x", FileFormat:=Excel.XlFileFormat.xlOpenXMLWorkbook)
        app.Workbooks.Close()
        'app.Quit()
        'Dim wb = app.Workbooks.Open(file)
        'wb.SaveAs(Filename:=file + "x", FileFormat:=Excel.XlFileFormat.xlOpenXMLWorkbook)
        'wb.Close()
        'app.Quit()
        Dim fi As FileInfo = New FileInfo(file)
        fi.Delete()
        Return $"{file}x"
    End Function

    ''' <summary>Holt Daten aus einer Excel-Datei und einem oder mehreren Worksheets und fügt diese in temporäre DTs via EPPlus</summary>
    ''' <param name="importFile">Die Datei, die Importiert werden soll</param>
    ''' <param name="QuelleNichtLoeschen"></param>
    ''' <param name="HeaderRow">Die Zeile in der Quelldatei, welche die Spaltenköpfe enthält</param>
    ''' <returns></returns>
    Public Function ExcelImportToTempDataTables(importFile As String, Optional QuelleNichtLoeschen As Boolean = False, Optional HeaderRow As Integer = 1) As List(Of DataTable)
        Dim lstDt As New List(Of DataTable)
        Dim fi As FileInfo = New FileInfo(importFile)
        If Not fi.Exists Then
            msgExclamation($"Datei {importFile} existiert nicht mehr. Zwischenzeitig gelöscht?!")
            Return Nothing
        End If
        If importFile Like "*.xls" Then
            importFile = ConvertXlsToXlsx(importFile) 'altes .xls-Format in aktuelles .xlsx-Format konvertieren
            fi = New FileInfo(importFile)
        End If
        Dim isProtected = False
        Dim FilePassword = ""
        If fi.FullName.IsInUse Then
            msgExclamation($"Die Datei {fi.Name} ist bereits geöffnet. Bitte schließen und erneut probieren.")
            Return Nothing
        End If
        WaitCursor()
        Try 'prüfen, ob Datei passwortgeschützt ist.
            Dim tryOpen = New ExcelPackage(fi)
            isProtected = False
        Catch ex As Exception
            isProtected = True
            Using dlg As New dlgTextInput
                dlg.Text = $"Passwort für Excel-Datei {fi.Name} eingeben..."
                dlg.ShowDialog()
                FilePassword = dlg.tbInput.Text
            End Using
            If FilePassword = "" Then Return Nothing
        End Try
        DefaultCursor()
        Try
            Using xlPackage As ExcelPackage = If(isProtected, New ExcelPackage(fi, True, FilePassword), New ExcelPackage(fi))
                Dim fileName = importFile.LastRightCut("\")
                Dim worksheets As New List(Of ExcelWorksheet)
                If xlPackage.Workbook.Worksheets.Count > 1 Then
                    worksheets = SelectExcelWorksheets(xlPackage)
                Else
                    If xlPackage.Compatibility.IsWorksheets1Based Then
                        worksheets.Add(xlPackage.Workbook.Worksheets(1))
                    Else
                        worksheets.Add(xlPackage.Workbook.Worksheets(1))
                    End If
                End If
                If worksheets.Count <> 0 Then
                    WaitCursor()
                    For Each wks In worksheets
                        Dim dt As New DataTable With {.TableName = wks.Name}
                        lstDt.Add(dt)
                        Dim startCell As ExcelCellAddress = wks.Dimension.Start
                        Dim endCell As ExcelCellAddress = wks.Dimension.[End]
                        'Cols
                        For col As Integer = startCell.Column To endCell.Column
                            Dim colText = wks.Cells(HeaderRow, col).Text 'Headertext anstelle Nummer
                            dt.Columns.Add(colText)
                        Next
                        'Rows
                        For row As Integer = startCell.Row + HeaderRow To endCell.Row 'Header nicht als Row werten
                            Dim dr As DataRow = dt.NewRow()
                            Dim x As Integer = 0
                            For col As Integer = startCell.Column To endCell.Column
                                dr(Math.Min(System.Threading.Interlocked.Increment(x), x - 1)) = wks.Cells(row, col).Value
                            Next
                            dt.Rows.Add(dr)
                        Next
                    Next
                    DefaultCursor()
                Else
                    msgExclamation("kein Tabellenblatt ausgewählt!")
                    Return Nothing
                End If
            End Using
            If Not QuelleNichtLoeschen Then
                If msgQuestionInfo("Soll die Quelldatei gelöscht werden") = DialogResult.Yes Then fi.Delete()
            End If
            Return lstDt
        Catch ex As Exception
            msgEx(ex)
            Return Nothing
        End Try
    End Function

    ''' <summary>
    ''' Holt Daten aus einer Excel-Datei und fügt diese in eine temporäre DT via EPPlus
    ''' </summary>
    ''' <param name="importFile">Die Datei, die Importiert werden soll</param>
    ''' <param name="QuelleNichtLoeschen"></param>
    ''' <param name="HeaderRow">Die Zeile in der Quelldatei, welche die Spaltenköpfe enthält</param>
    ''' <returns></returns>
    Public Function ExcelImportToTempDataTable(importFile As String, Optional QuelleNichtLoeschen As Boolean = False, Optional HeaderRow As Integer = 1) As DataTable
        Dim dtExcel As DataTable = New DataTable()
        dtExcel.Clear()
        Dim fi As FileInfo = New FileInfo(importFile)
        If Not fi.Exists Then
            msgExclamation($"Datei {importFile} existiert nicht mehr. Zwischenzeitig gelöscht?!")
            Return Nothing
        End If
        If importFile Like "*.xls" Then
            importFile = ConvertXlsToXlsx(importFile) 'altes .xls-Format in aktuelles .xlsx-Format konvertieren
            fi = New FileInfo(importFile)
        End If
        Dim isProtected = False
        Dim FilePassword = ""
        If fi.FullName.IsInUse Then
            msgExclamation($"Die Datei {fi.Name} ist bereits geöffnet. Bitte schließen und erneut probieren.")
            Return Nothing
        End If
        WaitCursor()
        Try 'prüfen, ob Datei passwortgeschützt ist.
            Dim tryOpen = New ExcelPackage(fi)
            isProtected = False
        Catch ex As Exception
            isProtected = True
            Using dlg As New dlgTextInput
                dlg.Text = $"Passwort für Excel-Datei {fi.Name} eingeben..."
                dlg.ShowDialog()
                FilePassword = dlg.tbInput.Text
            End Using
            If FilePassword = "" Then Return Nothing
        End Try
        DefaultCursor()
        Try
            Using xlPackage As ExcelPackage = If(isProtected, New ExcelPackage(fi, True, FilePassword), New ExcelPackage(fi))
                Dim fileName = importFile.LastRightCut("\")
                Dim worksheet As ExcelWorksheet
                If xlPackage.Workbook.Worksheets.Count > 1 Then
                    worksheet = xlPackage.Workbook.Worksheets(ChooseExcelWorksheet(xlPackage, fileName))
                    If worksheet Is Nothing Then Return Nothing
                Else
                    If xlPackage.Compatibility.IsWorksheets1Based Then
                        worksheet = xlPackage.Workbook.Worksheets(1)
                    Else
                        worksheet = xlPackage.Workbook.Worksheets(0)
                    End If
                End If
                If worksheet IsNot Nothing Then
                    WaitCursor()
                    Dim startCell As ExcelCellAddress = worksheet.Dimension.Start
                    Dim endCell As ExcelCellAddress = worksheet.Dimension.[End]
                    'Cols
                    For col As Integer = startCell.Column To endCell.Column
                        Dim colText = worksheet.Cells(HeaderRow, col).Text 'Headertext anstelle Nummer
                        dtExcel.Columns.Add(colText)
                    Next
                    'Rows
                    For row As Integer = startCell.Row + HeaderRow To endCell.Row 'Header nicht als Row werten
                        Dim dr As DataRow = dtExcel.NewRow()
                        Dim x As Integer = 0
                        For col As Integer = startCell.Column To endCell.Column
                            dr(Math.Min(System.Threading.Interlocked.Increment(x), x - 1)) = worksheet.Cells(row, col).Value
                        Next
                        dtExcel.Rows.Add(dr)
                    Next
                    DefaultCursor()
                Else
                    msgExclamation("kein Tabellenblatt ausgewählt!")
                    Return Nothing
                End If
            End Using
            If Not QuelleNichtLoeschen Then
                If msgQuestionInfo("Soll die Quelldatei gelöscht werden") = DialogResult.Yes Then fi.Delete()
            End If
            Return dtExcel
        Catch ex As Exception
            msgEx(ex)
            Return Nothing
        End Try
    End Function

    ''' <summary>Auswahl mehrerer Tabellenlätter</summary>
    Private Function SelectExcelWorksheets(xlPackage As ExcelPackage) As List(Of ExcelWorksheet)
        Dim lstSelectedWks As New List(Of ExcelWorksheet)
        Using dlg As New dlgChkLb
            dlg.Text = "Arbeitsblätter auswählen"
            xlPackage.Workbook.Worksheets.ForEach(Sub(wks) dlg.chkLb.Items.Add(wks.Name))
            If dlg.ShowDialog() <> DialogResult.OK Then Return Nothing
            For Each itm In dlg.chkLb.CheckedItems
                lstSelectedWks.Add(xlPackage.Workbook.Worksheets(itm.ToString))
            Next
        End Using
        Return lstSelectedWks
    End Function

    ''' <summary>Auswahl eines Excel-Tabellenblattes</summary>
    ''' <param name="xlPackage">Das aktuelle ExcelPackage</param>
    ''' <param name="Filename">Dateiname der Excel-Datei</param>
    ''' <returns>Den Namen des ausgewählten Tabellenblattes</returns>
    Private Function ChooseExcelWorksheet(xlPackage As ExcelPackage, Filename As String) As String
        Dim usedWks As String = ""
        Using dlg As New dlgListboxAuswahl
            Dim FileNameOnly = Path.GetFileName(Filename)
            dlg.Text = $"Tabellenblatt für Datei {FileNameOnly} auswählen..."
            For Each wks As ExcelWorksheet In xlPackage.Workbook.Worksheets
                dlg.lbAuswahl.Items.Add(wks.Name)
            Next
            If dlg.ShowDialog = DialogResult.OK Then
                If dlg.lbAuswahl.SelectedItem IsNot Nothing Then
                    usedWks = dlg.lbAuswahl.SelectedItem.ToString
                Else
                    usedWks = Nothing
                End If
            End If
        End Using
        Return usedWks
    End Function

    ''' <summary> Importiert Daten aus einer Exceldatei in die angegebene Tabelle im DataSet - nicht über EPPlus</summary>
    <Extension>
    Public Sub importExcelData(tbl As DataTable)
        Dim importFile = GetFileNameWithPath("Excel-Dateien (*.xlsx)|*.xlsx")
        If Not importFile = "" Then
            Using excelCon = New OleDbConnection("Provider = Microsoft.ACE.OLEDB.12.0; Data Source='" & importFile & "'; " & "Extended Properties='Excel 12.0;HDR=YES;IMEX=1;'")
                excelCon.Open()
                Using da = New OleDbDataAdapter("Select * From [Tabelle1$]", excelCon)
                    Dim tblSrc As New DataTable("ExcelSource")
                    da.Fill(tblSrc)
                    For Each rw As DataRow In tblSrc.Rows
                        Dim activeRow = $"{rw.Item("ID")}"
                        Try
                            tbl.ImportRow(rw)
                        Catch ex As Exception
                            If msgQuestionWarning($"FEHLER: {ex.Message} | {activeRow} {Environment.NewLine} abbrechen?") = DialogResult.Yes Then
                                Return
                            End If
                        End Try
                    Next
                End Using
            End Using
        Else
            Return
        End If
    End Sub

    ''' <summary>Überprüft, ob die angegebenen Spaltennamen in einer DataTable vorhanden sind</summary>
    ''' <param name="dt">Die zu prüfende DataTable</param>
    ''' <param name="lstColumnName">Eine Liste mit Spaltennamen</param>
    ''' <returns>True wenn alle Spaltennamen vorhanden sind, False wenn ein Spaltenname nicht vorhanden ist</returns>
    Public Function EnsureValidHeaders(dt As DataTable, lstColumnName As List(Of String)) As Boolean
        Dim valid = False
        For Each columnName In lstColumnName
            If dt.ColumnsX.Any(Function(x) x.ColumnName = columnName) Then
                valid = True
                Exit For
            Else
                valid = False
            End If
            If Not valid Then
                msgInformation($"Die Spalte {columnName} konnte im Excel-Dokument nicht gefunden werden!")
                Return False
            End If
        Next
        Return True
    End Function

#End Region 'Import

#End Region 'Excel

#Region "Outlook"

    'TODO: probieren, String() ist unangenehm in der Übermittlung, wenn man z.B. nur einen Empfänger hat
    Public Sub SendMailViaOutlook(Recipients As String(), CarbonCopy As String(), BlindCarbonCopy As String(), Subject As String, Salutation As String, Text As String(), Signature As String, AttachmentFiles As String(), SendMail As Boolean)
        Dim olApp As New Outlook.Application
        Dim mail = DirectCast(olApp.CreateItem(Outlook.OlItemType.olMailItem), Outlook.MailItem)
        With mail
            .To = String.Join("; ", Recipients)
            .CC = String.Join("; ", CarbonCopy)
            .BCC = String.Join("; ", BlindCarbonCopy)
            .Subject = Subject
            .HTMLBody = $"<font face=calibri>{Salutation}<BR><BR>{String.Join("<BR><BR>", Text)}<BR><BR>{Signature}</font>"
            If AttachmentFiles.Count > 0 Then
                AttachmentFiles.ForEach(Sub(File) .Attachments.Add(File))
            End If
            If SendMail Then
                If Not .To = "" Then
#If Not DEBUG Then
                    .Send()
#Else
                    .Display()
#End If
                Else
                    .Display()
                End If
            Else
                .Display()
            End If
        End With
    End Sub

    ''' <summary> verschickt eine E-Mail Optional mit Anhang, Optional mit CC-Empfänger über Microsoft Outlook </summary>
    Public Function OutlookSendMail(AN As String, Betreff As String, Anrede As String, Text1 As String, Text2 As String, Signatur As String, Automail As Boolean, Optional Anhang As String = "", Optional CC As String = "", Optional Anhaenge As List(Of String) = Nothing) As Boolean
        Try
            Dim olApp As New Outlook.Application
            Dim mail = DirectCast(olApp.CreateItem(Outlook.OlItemType.olMailItem), Outlook.MailItem)
            With mail
                .To = AN
                If Not CC = "" Then .CC = CC
                .Subject = Betreff
                .HTMLBody = If(Text2 = "", $"<font face=calibri>{Anrede}<BR><BR>{Text1}<BR><BR>{Signatur}</font>", $"<font face=calibri>{Anrede}<BR><BR>{Text1}<BR><BR>{Text2}<BR><BR>{Signatur}</font>")
                If Not Anhang = "" Then .Attachments.Add(Anhang)
                If Anhaenge IsNot Nothing Then
                    For Each att In Anhaenge
                        .Attachments.Add(att)
                    Next
                End If
                If Automail Then
                    If Not AN = "" Then
                        .Send()
                    Else
                        .Display()
                    End If
                Else
                    .Display()
                End If
            End With
            Return True
        Catch ex As Exception
            msgError(ex.Message)
            Return False
        End Try
    End Function

    ''' <summary> bereitet eine E-Mail nur mit Anhängen vor </summary>
    Public Sub OutlookSendMail(Attachments As List(Of String))
        Dim olApp As New Outlook.Application
        Dim mail = DirectCast(olApp.CreateItem(Outlook.OlItemType.olMailItem), Outlook.MailItem)
        With mail
            Try
                For Each att In Attachments
                    .Attachments.Add(att)
                Next
            Catch ex As Runtime.InteropServices.COMException
                msgEx(ex)
            End Try
            .Display()
        End With
    End Sub

#End Region 'Outlook

    ''' <summary> Fügt einen String in die Zwischenablage ein</summary>
    <Extension>
    Public Sub setClipBoardData(Text As String)
        Dim dObject As New DataObject
        With dObject
            .SetData(DataFormats.Rtf, False, Text)
            .SetData(DataFormats.UnicodeText, False, Text)
        End With
        Clipboard.SetDataObject(dObject)
    End Sub

#Region "PDF (mit MigraDoc)"

    'Private Sub exportSelectionToPDF(dgv As DataGridView, pdfDescription As String, selectedCols As List(Of dgvCol), Header As Boolean, Optional lstrow As List(Of DataGridViewRow) = Nothing)
    '    Dim pdfFile = saveAs("PDF-Dateien|*.pdf")
    '    If pdfFile = "" Then Return
    '    If File.Exists(pdfFile) Then File.Delete(pdfFile)
    '    Dim dtExport As New DataTable
    '    For Each col In selectedCols
    '        Dim dtCol As New DataColumn With {.ColumnName = col.name, .Caption = col.headertext}
    '        dtExport.Columns.Add(dtCol)
    '    Next
    '    If lstrow IsNot Nothing Then
    '        For Each rw As DataGridViewRow In lstrow
    '            dtExport.Rows.Add()
    '            For Each cell As DataGridViewCell In rw.Cells
    '                Dim colName = cell.OwningColumn.Name
    '                If colExist(dtExport, colName) Then dtExport.Rows(dtExport.Rows.Count - 1)(colName) = CStr(cell.FormattedValue)
    '            Next
    '        Next
    '        'showDebugDGV(dtExport)
    '    Else
    '        For Each rw As DataGridViewRow In dgv.Rows
    '            dtExport.Rows.Add()
    '            For Each cell As DataGridViewCell In rw.Cells
    '                If cell.OwningColumn.Visible = True Then
    '                    Dim colName = cell.OwningColumn.Name
    '                    If colExist(dtExport, colName) Then dtExport.Rows(dtExport.Rows.Count - 1)(colName) = CStr(cell.FormattedValue)
    '                End If
    '            Next
    '        Next
    '    End If

    '    Dim document As Document = CreatePdfDoc(pdfDescription, dtExport)
    '    document.UseCmykColor = True
    '    Dim pdfRenderer As PdfDocumentRenderer = New PdfDocumentRenderer(True)
    '    pdfRenderer.Document = document
    '    pdfRenderer.RenderDocument()
    '    pdfRenderer.Save(pdfFile)
    '    If msgQuestionInfo($"Der Export wurde unter {pdfFile} gespeichert.{Environment.NewLine}Soll die Datei geöffnet werden?") = DialogResult.Yes Then
    '        Process.Start(pdfFile)
    '    End If
    'End Sub

    'Private Function CreatePdfDoc(description As String, dt As DataTable) As MigraDoc.DocumentObjectModel.Document
    '    Dim pdfDoc = New Document()
    '    pdfDoc.Info.Title = description
    '    pdfDoc.DefaultPageSetup.Orientation = Orientation.Landscape
    '    PdfDocDefineStyles(pdfDoc)
    '    Dim docTable = PdfDocCreatePage(pdfDoc, dt, description)
    '    PdfDocFillContent(docTable, dt)
    '    Return pdfDoc
    'End Function

    'Private Sub PdfDocDefineStyles(doc As Document)
    '    Dim style As MigraDoc.DocumentObjectModel.Style = doc.Styles("Normal")
    '    style.Font.Name = "Arial"
    '    style = doc.Styles(StyleNames.Header)
    '    style.ParagraphFormat.AddTabStop("16cm", TabAlignment.Right)
    '    style = doc.Styles(StyleNames.Footer)
    '    style.ParagraphFormat.AddTabStop("8cm", TabAlignment.Center)
    '    style = doc.Styles.AddStyle("Table", "Normal")
    '    style.Font.Name = "Arial"
    '    style.Font.Size = 9
    '    style = doc.Styles.AddStyle("Reference", "Normal")
    '    style.ParagraphFormat.SpaceBefore = "5mm"
    '    style.ParagraphFormat.SpaceAfter = "5mm"
    '    style.ParagraphFormat.TabStops.AddTabStop("16cm", TabAlignment.Right)
    'End Sub

    'Private Function PdfDocCreatePage(doc As Document, dt As DataTable, footertext As String) As Tables.Table
    '    Dim section As Section = doc.AddSection()
    '    Dim paragraph As Paragraph = section.Footers.Primary.AddParagraph()
    '    paragraph.AddText(footertext)
    '    paragraph.Format.Font.Size = 9
    '    paragraph.Format.Alignment = ParagraphAlignment.Center
    '    Dim docTable = section.AddTable()
    '    docTable.Style = "Table"
    '    'docTable.Borders.Color = Tables.
    '    docTable.Borders.Width = 0.25
    '    docTable.Borders.Left.Width = 0.5
    '    docTable.Borders.Right.Width = 0.5
    '    docTable.Rows.LeftIndent = 0
    '    Dim column As MigraDoc.DocumentObjectModel.Tables.Column
    '    For Each col As DataColumn In dt.Columns
    '        column = docTable.AddColumn(Unit.FromCentimeter(3))
    '        column.Format.Alignment = ParagraphAlignment.Center
    '    Next
    '    Dim row As Tables.Row = docTable.AddRow()
    '    row.HeadingFormat = True
    '    row.Format.Alignment = ParagraphAlignment.Center
    '    row.Format.Font.Bold = True
    '    'row.Shading.Color = TableBlue
    '    For i As Integer = 0 To dt.Columns.Count - 1
    '        row.Cells(i).AddParagraph(dt.Columns(i).ColumnName)
    '        row.Cells(i).Format.Font.Bold = False
    '        row.Cells(i).Format.Alignment = ParagraphAlignment.Left
    '        row.Cells(i).VerticalAlignment = Tables.VerticalAlignment.Bottom
    '    Next
    '    docTable.SetEdge(0, 0, dt.Columns.Count, 1, Tables.Edge.Box, BorderStyle.Single, 0.75, Color.Empty)
    '    Return docTable
    'End Function

    'Private Sub PdfDocFillContent(docTable As Tables.Table, dt As DataTable)
    '    Dim row1 As Tables.Row
    '    For i As Integer = 0 To dt.Rows.Count - 1
    '        row1 = docTable.AddRow()
    '        row1.TopPadding = 1.5
    '        For j As Integer = 0 To dt.Columns.Count - 1
    '            'row1.Cells(j).Shading.Color = TableGray
    '            row1.Cells(j).VerticalAlignment = Tables.VerticalAlignment.Center
    '            row1.Cells(j).Format.Alignment = ParagraphAlignment.Left
    '            row1.Cells(j).Format.FirstLineIndent = 1

    '            row1.Cells(j).AddParagraph(dt.Rows(i)(j).ToString())
    '            docTable.SetEdge(0, docTable.Rows.Count - 2, dt.Columns.Count, 1, Tables.Edge.Box, BorderStyle.Single, 0.75)
    '        Next
    '    Next
    'End Sub

#End Region 'PDF 'TODO: passt sich nicht an Zellinhalt/Seite an....

End Module

Public Class OutlookMail

    Property Attachments As List(Of String)
    Property Recipients As List(Of String)
    Property CarbonCopy As List(Of String)
    Property BlindCarbonCopy As List(Of String)
    Property Subject As String
    Property Salutation As String
    Property Text As List(Of String)
    Property Signature As String
    Property SendMail As Boolean

    Public Sub New()
        Dim olApp As New Outlook.Application
        Dim mail = DirectCast(olApp.CreateItem(Outlook.OlItemType.olMailItem), Outlook.MailItem)

    End Sub

    Private Sub CreateAndSendOrShowMail(mail As Outlook.MailItem)
        With mail
            .To = String.Join("; ", Recipients)
            .CC = String.Join("; ", CarbonCopy)
            .BCC = String.Join("; ", BlindCarbonCopy)
            .Subject = Subject
            .HTMLBody = $"<font face=calibri>{Salutation}<BR><BR>{String.Join("<BR><BR>", Text)}<BR><BR>{Signature}</font>"
            If Not Attachments.Count = 0 Then
                For Each File In Attachments
                    .Attachments.Add(File)
                Next
            End If
            If SendMail Then
                If Not .To = "" Then
                    .Send()
                Else
                    .Display()
                End If
            Else
                .Display()
            End If
        End With
    End Sub

End Class
